#include "mlsRec.h"
#include <cmath>
#include <Eigen/Dense>

using namespace std;
using namespace Eigen;

QImage MLSRec::apply(const QImage &img,float sigma) {

  // TODO apply weighted least squares on each pixel of the input image
  // example of image manipulation using Qt: 

  // copy input 
  QImage newImg = img; 

  // iterate eover pixels 
  for(int x=0;x<img.width();++x) {
    for(int y=0;y<img.height();++y) {
      // get color at x,y
      Color current = getColor(img,x,y);
      
      // check if this color is missing in the image
      if(colorMissing(current)) {
	// set color as red 
	current = Color(255.0f,0.0f,0.0f,255.0f);
      }
      // set color in the new image 
      setColor(newImg,x,y,current);
    }

    // show progression 
    if(!(x%10))
      cout << "MLS: " << (int)(((float)x/(float)(img.width()-2))*100.0f) << " percent done..." << '\r' << flush;
  }


  // example of matrix manip using Eigen:
  
  // 0 matrix with 4 rows, 3 columns
  MatrixXf M = MatrixXf::Zero(4,3);
  
  // set row 1 and column 3 to 5.0f
  M(0,2) = 5.0f;
  
  // transposed matrix 
  MatrixXf T = M.transpose();
  
  // multiplication 
  MatrixXf MT = M*T;

  // matrix inversion
  MatrixXf MTi = MT.inverse();

  // vector of size 4
  VectorXf v = VectorXf::Zero(4); 

  // matrix vector multiplication
  VectorXf Mv = MTi*v;

  return newImg;
}

MLSRec::Color MLSRec::estimateColorPlane(const QImage &img,int x,int y,float sigma) {
  // estimate (and eval) a,b,c such that 
  // ax + by + c = z
  // z being the value at pixel x,y (One MLS per channel then!)

  // TODO
  return Color(0.0f,0.0f,0.0f,255.0f);
}

MLSRec::Color MLSRec::estimateColorQuadric(const QImage &img,int x,int y,float sigma) {
  // estimate (and eval) a,b,c,d,e,f such that 
  // ax^2 + by^2 + cxy + dx + ey + f = z
  // z being the value at pixel x,y (One MLS per channel then!)

  // TODO
  return Color(0.0f,0.0f,0.0f,255.0f);
}


float MLSRec::estimateSigma(const QImage &img,int x,int y) {
  // estimate sigma at x,y depending on the positions of nearest valid neighbors
  
  // TODO 
  return 2.0f;
}
